/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "AudioCapture.h"
#include "WaveFile.hpp"

AudioCaptureData::AudioCaptureData()
  : length(0), nChannels(0), sampleRate(0), sampleSize(0)
{}

const AudioCaptureData &AudioCaptureData::operator=(const AudioCaptureData &fmt)
{
	path = fmt.path;
	length = fmt.length;
	nChannels = fmt.nChannels; 
	sampleRate = fmt.sampleRate;
	sampleSize = fmt.sampleSize;
	return(*this);
}

int AudioCaptureData::blockAlign() const
{
	return(nChannels * sampleSize/8);
}

int AudioCaptureData::byteRate() const
{
	return(nChannels * sampleRate * sampleSize/8);
}

int AudioCaptureData::nTotalBytes() const
{
	return(length * nChannels * sampleRate * sampleSize/8);
}

/*******************************************************************************************/

CAudioCapture::CAudioCapture()
  : _pDSC(0), _pDSCB(0)
{
	_dscbd.dwSize = sizeof(DSCBUFFERDESC);       //initialize capture properties
	_dscbd.dwFlags = 0;
	_dscbd.dwReserved = 0;
	_dscbd.lpwfxFormat = &_waveCaptureFmt;
	_dscbd.dwFXCount = 0;
	_dscbd.lpDSCFXDesc = NULL;

	_waveCaptureFmt.cbSize = 0;
	_waveCaptureFmt.wFormatTag = WAVE_FORMAT_PCM;
}

CAudioCapture::~CAudioCapture()
{
	if(_pDSC) {
		_pDSCB->Release();
		_pDSC->Release();
	}
}

bool CAudioCapture::Initialize(const AudioCaptureData &format)
{
	if(DirectSoundCaptureCreate8(0, &_pDSC, 0) != DS_OK)
		return(false);

	_dscbd.dwBufferBytes = format.nTotalBytes(); //set capture properties
	_waveCaptureFmt.nAvgBytesPerSec = format.byteRate();
	_waveCaptureFmt.nBlockAlign = format.blockAlign();
	_waveCaptureFmt.nChannels = format.nChannels;
	_waveCaptureFmt.nSamplesPerSec = format.sampleRate;
	_waveCaptureFmt.wBitsPerSample = format.sampleSize;

	if(_pDSC->CreateCaptureBuffer(&_dscbd, &_pDSCB, NULL) == DS_OK)
		return(true);
	else return(false);
}

void CAudioCapture::Start()
{
	if(_pDSCB) _pDSCB->Start(0);
}

void CAudioCapture::Stop()
{
	if(_pDSCB) _pDSCB->Stop();
}

bool CAudioCapture::WriteData(const std::string &path, const std::string &identifier)
{
	std::string fileName = path;
	DWORD bufLen, readPos;
	void *buf = 0;

	if(path.empty()) return(false);

	if(path[path.length()-1] != '\\')            //construct wave file name
		fileName.push_back('\\');
	fileName.append(identifier);
	fileName.append(".wav");

	if(_pDSCB->GetCurrentPosition(0, &readPos) != DS_OK)
		return(false);

	if(readPos == 0)
		readPos = _dscbd.dwBufferBytes;

	_pDSCB->Lock(0, readPos, &buf, &bufLen, 0, 0, 0);

	CWaveFile wave(fileName, false);             //write wave data
	wave.writeHeader(&_waveCaptureFmt);
	wave.writeWaveData(buf, bufLen);
	wave.close();

	_pDSCB->Unlock(buf, bufLen, 0, 0);
	return(true);
}
