/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "AudioRender.h"
#include "EyeTrack.h"
#include "WaveFile.hpp"

/****************************************************************************/

AudioRenderData::AudioRenderData()
	: startDelay(0), stopDelay(0)
{}

const AudioRenderData& AudioRenderData::operator=(const AudioRenderData &rhs)
{
	path = rhs.path;
	startDelay = rhs.startDelay;
	stopDelay = rhs.stopDelay;
	return(*this);
}

/****************************************************************************/

CAudioRender::CAudioRender()
	: _pDS(0), _pDSB(0), _data(), _audioOffTime(0), 
	  _audioOnTime(static_cast<unsigned int>(-1)), 
	  _length(0), _stopTime(0), _isPlaying(false)
{
	_dsbd.dwSize = sizeof(DSBUFFERDESC);         //initialize render properties
	_dsbd.dwFlags = 0;
	_dsbd.dwReserved = 0;
	_dsbd.lpwfxFormat = &_waveRenderFmt;
	_dsbd.guid3DAlgorithm = DS3DALG_DEFAULT;

	_waveRenderFmt.cbSize = 0;
	_waveRenderFmt.wFormatTag = WAVE_FORMAT_PCM;
}

CAudioRender::~CAudioRender()
{
	if(_pDS) {
		_pDSB->Release();
		_pDS->Release();		
	}
}

unsigned int CAudioRender::AudioOffTime() const
{
	return(_audioOffTime);
}

unsigned int CAudioRender::AudioOnTime() const
{
	return(_audioOnTime);
}

unsigned int CAudioRender::AudioStartDelay() const
{
	return(_data.startDelay);
}

unsigned int CAudioRender::AudioStopDelay() const
{
	return(_data.stopDelay);
}

bool CAudioRender::Initialize(const AudioRenderData &data)
{
	CWaveFile wave(data.path, true);
	DWORD bufLen;
	void *buf = 0;

	if(DirectSoundCreate8(0, &_pDS, 0) != DS_OK)
		return(false);

	_pDS->SetCooperativeLevel(theApp.GetMainWnd()->GetSafeHwnd(), DSSCL_PRIORITY);
	wave.readHeader(&_waveRenderFmt);            //set render properties
	_dsbd.dwBufferBytes = _waveRenderFmt.nBlockAlign * wave.nSamples();
	_length = static_cast<unsigned int>((float(wave.nSamples()) / 
										 float(_waveRenderFmt.nSamplesPerSec)) * 1000.0) + 1;

	if(_pDS->CreateSoundBuffer(&_dsbd, &_pDSB, 0) != DS_OK)
		return(false);

	_pDSB->Lock(0, 0, &buf, &bufLen, 0, 0, DSBLOCK_ENTIREBUFFER);
	wave.readWaveData(buf, bufLen);              //load wave data into buffer
	wave.close();
	_pDSB->Unlock(buf, bufLen, 0, 0);
	_pDSB->SetCurrentPosition(0);
	_pDSB->SetVolume(DSBVOLUME_MAX);

	_data = data;
	return(true);
}

CAudioRender::PollResult CAudioRender::Poll(unsigned int time)
{
	if(_pDSB && time >= _audioOnTime) {
		if(time >= _audioOffTime && _isPlaying) {
			_pDSB->Stop();
			_isPlaying = false;
			return(RENDER_STOP);
		}
		else if(time < _audioOffTime && !_isPlaying) {
			_pDSB->Play(0, 0, 0);
			_isPlaying = true;
			return(RENDER_START);
		}
	}
	return(RENDER_UNCHANGED);
}

void CAudioRender::Start(unsigned int time)
{
	_audioOnTime = time + _data.startDelay;
	_audioOffTime = _audioOnTime + _length;
	_stopTime = _audioOffTime + _data.stopDelay;
}

void CAudioRender::Stop()
{
	if(_pDSB) _pDSB->Stop();
	_isPlaying = false;
}

unsigned int CAudioRender::StopTime() const
{
	return(_stopTime);
}
