/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "DirectInput.h"
#include "EyeTrack.h"

CDirectInput::CDirectInput()
	: _pDI(0), _pKeyboard(0), _pMouse(0), _cursorLoc(0, 0), _ready(false)
{}

CDirectInput::~CDirectInput()
{
	Terminate();
}


const CPoint &CDirectInput::GetCursorLocation()
{
	DIMOUSESTATE diMouseState;

	if(_pMouse->GetDeviceState(sizeof(diMouseState), &diMouseState) == DI_OK)
		_cursorLoc.Offset(diMouseState.lX, diMouseState.lY);
	return(_cursorLoc);
}

bool CDirectInput::Initialize()
{
	DIPROPDWORD dipdw;
	HRESULT hr = DirectInput8Create(theApp.m_hInstance, DIRECTINPUT_VERSION, 
									IID_IDirectInput8, (void**)&_pDI, NULL); 

	dipdw.diph.dwSize = sizeof(DIPROPDWORD);     //set buffer properties
	dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
	dipdw.diph.dwObj = 0;
	dipdw.diph.dwHow = DIPH_DEVICE;

	if(FAILED(hr)) {                             //failed to create Direct Input instance
		_pDI = 0;
		TRACE("DirectInput8Create failed.\n");
		return(false);
	}
                                                 /*** Initialize DIRECT KEYBOARD ***/
	hr = _pDI->CreateDevice(GUID_SysKeyboard, &_pKeyboard, NULL); 
	if(FAILED(hr)) {                             //failed to initialize device
		TRACE("CreateDevice KBD failed.\n");
		Terminate();
		return(false);
	}

	hr = _pKeyboard->SetDataFormat(&c_dfDIKeyboard); 
	if(FAILED(hr)) {                             //failed to set sharing parameters
		TRACE("SetDataFormat KBD failed.\n");
		Terminate();
		return(false);
	}

	hr = _pKeyboard->SetCooperativeLevel(theApp.GetMainWnd()->GetSafeHwnd(), 
										 DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);
	if(FAILED(hr)) {                             //failed to set cooperative level
		TRACE("SetCooperativeLevel KBD failed.\n");
		Terminate();
		return(false);
	}

	hr = _pKeyboard->Acquire();
	if(FAILED(hr)) {                             //failed to acquire keyboard device
		TRACE("Acquire KBD failed.\n");
		Terminate();
		return(false);
	}
				                                 /*** Initialize DIRECT MOUSE ***/
	hr = _pDI->CreateDevice(GUID_SysMouse, &_pMouse, NULL); 
	if(FAILED(hr)) {                             //failed to initialize mouse device
		TRACE("CreateDevice MOUSE failed.\n");
		Terminate();
		return(false);
	}

	hr = _pMouse->SetDataFormat(&c_dfDIMouse);
	if(FAILED(hr)) {                             //failed to set mouse format
		TRACE("SetDataFormat MOUSE failed.\n");
		Terminate();
		return(false);
	}

	hr = _pMouse->SetCooperativeLevel(theApp.GetMainWnd()->GetSafeHwnd(),
									  DISCL_NONEXCLUSIVE | DISCL_BACKGROUND);
	if(FAILED(hr)) {                             //failed to set cooperative level
		TRACE("SetCooperativeLevel MOUSE failed.\n");
		Terminate();
		return(false);
	}

	dipdw.dwData = DIPROPAXISMODE_REL;
	hr = _pMouse->SetProperty(DIPROP_AXISMODE, &dipdw.diph);
	if(FAILED(hr)) {                             //failed to set mouse AXIZ mode
		TRACE("SetProperty MOUSE axis failed.\n");
		Terminate();
		return(false);
	}

	dipdw.dwData = 100;
	hr = _pMouse->SetProperty(DIPROP_BUFFERSIZE, &dipdw.diph);
	if(FAILED(hr)) {                             //failed to set device properties
		TRACE("SetProperty MOUSE buffer size failed.\n");
		Terminate();
		return(false);
	}

	hr = _pMouse->Acquire();
	if(FAILED(hr)) {                             //failed to acquire mouse device
		TRACE("Acquire MOUSE failed.\n");
		Terminate();
		return(false);
	}

	GetCursorPos(&_cursorLoc);
	_ready = true;
	return(true);
}

bool CDirectInput::IsLButtonDown(CPoint &pt)
{
	DIMOUSESTATE diMouseState;

	if(_pMouse->GetDeviceState(sizeof(diMouseState), &diMouseState) == DI_OK) {
		_cursorLoc.Offset(diMouseState.lX, diMouseState.lY);
		pt = _cursorLoc;
		if(diMouseState.rgbButtons[0] & 0x80)
			return(true);
	}
	return(false);
}

bool CDirectInput::IsRButtonDown(CPoint &pt)
{
	DIMOUSESTATE diMouseState;

	if(_pMouse->GetDeviceState(sizeof(diMouseState), &diMouseState) == DI_OK) {
		_cursorLoc.Offset(diMouseState.lX, diMouseState.lY);
		pt = _cursorLoc;
		if(diMouseState.rgbButtons[1] & 0x80)
			return(true);
	}
	return(false);
}

bool CDirectInput::IsReady() const
{
	return(_ready);
}

int CDirectInput::KeyDown()
{
    HRESULT hr;
	char buf[256];
	                                             /*** Read KEYBOARD Event ***/
    hr = _pKeyboard->GetDeviceState(sizeof(buf), static_cast<void*>(&(buf))); 
	if(FAILED(hr)) { 
		if(hr == DIERR_INPUTLOST)
			hr = _pKeyboard->Acquire();
		if(FAILED(hr))
			_ready = false;
	}
	else {
		for(int i=0; i < KEY_BUF_SIZE; ++i)
			if(buf[i] & 0x80)
				return(i);
	}
	return(-1);
}

void CDirectInput::Terminate()
{ 
	if(_pDI) { 
		if(_pKeyboard) {                         //disconnet direct keyboard
			_pKeyboard->Unacquire();
			_pKeyboard->Release();
			_pKeyboard = 0;
		}

		if(_pMouse) {                            //disconnet direct mouse
			_pMouse->Unacquire();
			_pMouse->Release();
			_pMouse = 0;
		}

		_pDI->Release();                         //shut down direct input
		_pDI = 0;
		_ready = false;
    } 
}
