/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include <sstream>

#include "EyeTrack.h"
#include "EditTrialDialog.h"
#include "EditTrialAudioDialog.h"
#include "EditTrialDisplayDialog.h"
#include "EditTrialPropertiesDialog.h"
#include "TrialWnd.h"
#include "EditTrialImagesDialog.h"
#include ".\edittrialdialog.h"

IMPLEMENT_DYNAMIC(CEditTrialDialog, CDialog)

CEditTrialDialog::CEditTrialDialog(CWnd* pParent, const Experiment *expt)
	: CDialog(CEditTrialDialog::IDD, pParent), _expt(expt), _editDisplayDlg(0),
	_answer(Tokenizer::NO_BUTTON), _displayTime(0), _controlType(Tokenizer::UNKNOWN_CONTROLLER)
{}

CEditTrialDialog::~CEditTrialDialog()
{}

void CEditTrialDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}


BEGIN_MESSAGE_MAP(CEditTrialDialog, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_CHANGE_AUDIO, OnBnClickedChangeAudio)
	ON_BN_CLICKED(IDC_BUTTON_CHANGE_DISPLAY, OnBnClickedChangeDisplay)
	ON_BN_CLICKED(IDC_BUTTON_CHANGE_PROPERTIES, OnBnClickedChangeProperties)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
END_MESSAGE_MAP()

Trial CEditTrialDialog::GetTrial() const
{
	Trial t(_identifier, _typeName, _display, _answer, _displayTime, _controlType);
	t.audioData() = _audio;
	return(t);
}

void CEditTrialDialog::OnBnClickedChangeAudio()
{
	CEditTrialAudioDialog dlg(this);

	dlg.RenderData() = _audio;
	if(dlg.DoModal() == IDOK) {
		_audio = dlg.RenderData();

		UpdateAudioData();
	}


}

void CEditTrialDialog::OnBnClickedChangeDisplay()
{
	CTrialWnd *tWnd = new CTrialWnd();
	CBrush brush(RGB(0, 0, 0));
	CString wndClass = AfxRegisterWndClass(CS_BYTEALIGNWINDOW | CS_OWNDC | CS_HREDRAW | CS_VREDRAW, 0, brush, 0);
	int height = ::GetSystemMetrics(SM_CYSCREEN);
	int width = ::GetSystemMetrics(SM_CXSCREEN);

	tWnd->SetWindowColors(_expt->backgroundColor(), _expt->foregroundColor());
	tWnd->CreateEx(WS_EX_APPWINDOW, wndClass, "Trial Preview", WS_VISIBLE | WS_POPUPWINDOW,
				  0, 0, width, height, this->GetSafeHwnd(), NULL);

	//User must be able to interact with both EditTrialDisplayDialog and TrialWnd
	//at same time, so cannot use modal dialog. When EditTrialDisplayDialog::OnOK
	//is clicked, EditTrialDialog::OnEditDisplayDialogOkay is called so data can be
	//passed back.  A bit convoluted, but it works.

	CEditTrialDisplayDialog *dlg = new CEditTrialDisplayDialog();
	TrialType *type = _expt->getTrialType(_typeName);

	dlg->Create(IDD_EDIT_TRIAL_DISPLAY_DIALOG, tWnd);
	tWnd->setEditDisplayCwnd(dlg);
	if(type->doGazeContingentStart())
		dlg->EnableGCRect();
	if(type->textFormat() != 0)
		dlg->EnableText(*(type->textFormat()));
	dlg->Initialize(_display);

	_editDisplayDlg = dlg;
}

void CEditTrialDialog::OnEditDisplayDialogOkay()
{
	CEditTrialDisplayDialog *dlg = (CEditTrialDisplayDialog*)_editDisplayDlg;

	_display = dlg->GetDisplayData();
	UpdateDisplayData();
}

void CEditTrialDialog::OnBnClickedChangeProperties()
{
	CEditTrialPropertiesDialog dlg(this);

	dlg.trialId() = _identifier;
	dlg.typeName() = _typeName;
	dlg.maxDisplayTime() = _displayTime;
	dlg.controlType() = _controlType;
	dlg.questionAnswer() = _answer;
	dlg.SetTrialTypeNames(_expt->getTrialTypeNames());

	Tokenizer::ControlType ctrl = (_expt->getControlType());

	dlg.controlType() = ctrl;
	

	if(dlg.DoModal() == IDOK) {
		_identifier = dlg.trialId();
		_typeName = dlg.typeName();
		_displayTime = dlg.maxDisplayTime();
		_answer = dlg.questionAnswer();
		_controlType = dlg.controlType();
		UpdatePropertyData();
	}
}

BOOL CEditTrialDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	UpdateAudioData();
	UpdateDisplayData();
	UpdatePropertyData();

	return TRUE;
}

void CEditTrialDialog::OnOK()
{
	if(_identifier.condition() == 0) {
		MessageBox("Trial properties not set.", "Trial Error");
		return;
	}

	if(_display.display1.empty()) {
		MessageBox("Trial display data not set.", "Trial Error");
		return;
	}

	CDialog::OnOK();
}

void CEditTrialDialog::SetTrial(const Trial *t)
{
	if(t) {
		_audio = const_cast<Trial*>(t)->audioData();   //remove "const" descriptor (member does not modify t)
		_display = t->displayData();
		_identifier = t->identifier();
		_typeName = t->typeId();
		_answer = t->questionAnswer();
		_displayTime = t->maxDisplayTime();
		_controlType = t->controlType();
	}
}

void CEditTrialDialog::UpdateAudioData()
{
	CWnd *ctrl = GetDlgItem(IDC_STATIC_AUDIO_FILE);
	char buf[10];

	if(!_audio.path.empty()) {                   //update path to audio file
		std::string::size_type loc = _audio.path.rfind('\\');

		if(loc == std::string::npos)
			ctrl->SetWindowText(_audio.path.c_str());
		else {
			std::string str("...");
			str.append(_audio.path.substr(loc));
			ctrl->SetWindowText(str.c_str());
		}
	}
	else
		ctrl->SetWindowText("<none>");

	ctrl = GetDlgItem(IDC_STATIC_START_DELAY);   //update audio start time
	ctrl->SetWindowText(_itoa(_audio.startDelay, buf, 10));

	ctrl = GetDlgItem(IDC_STATIC_STOP_DELAY);    //update audio stop time
	ctrl->SetWindowText(_itoa(_audio.stopDelay, buf, 10));
}

void CEditTrialDialog::UpdateDisplayData()
{
	std::ostringstream os;

	if(_display.display1.empty())
		GetDlgItem(IDC_STATIC_DISPLAY_STIMULUS1)->SetWindowText("<none>");
	else
		GetDlgItem(IDC_STATIC_DISPLAY_STIMULUS1)->SetWindowText(_display.display1.c_str());

	if(_display.display2.empty())
		GetDlgItem(IDC_STATIC_DISPLAY_STIMULUS2)->SetWindowText("<none>");
	else
		GetDlgItem(IDC_STATIC_DISPLAY_STIMULUS2)->SetWindowText(_display.display2.c_str());

	if(_display.isFixedDisplay)
		GetDlgItem(IDC_STATIC_DISPLAY_TYPE)->SetWindowText("fixed");
	else
		GetDlgItem(IDC_STATIC_DISPLAY_TYPE)->SetWindowText("display change");

	os<<'('<<_display.gcRect.top<<' '<<_display.gcRect.left<<' ';
	os<<_display.gcRect.bottom<<' '<<_display.gcRect.right<<')';
	GetDlgItem(IDC_STATIC_DISPLAY_GCRECT)->SetWindowText(os.str().c_str());
}

void CEditTrialDialog::UpdatePropertyData()
{
	CWnd *ctrl = GetDlgItem(IDC_STATIC_TRIAL_IDENTIFIER);
	char buf[10];
                                             //update trial identifier
	if(_identifier.condition() > 0 && _identifier.item() > 0)
		ctrl->SetWindowText(_identifier.idString().c_str());
	else ctrl->SetWindowText("<none>");
                                             //update trial type identifier
	ctrl = GetDlgItem(IDC_STATIC_TRIAL_CONDITION);
	if(!_typeName.empty())
		ctrl->SetWindowText(_typeName.c_str());
	else ctrl->SetWindowText("<none>");
                                             //update max display time
	ctrl = GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_TIME);
	ctrl->SetWindowText(_itoa(_displayTime, buf, 10));
                                             //update question answer
	ctrl = GetDlgItem(IDC_STATIC_TRIAL_ANSWER);
	if (_controlType == Tokenizer::FIVE_BUTTON){
		switch(_answer) {
			case Tokenizer::NO_BUTTON: ctrl->SetWindowText("<none>");      break;
			case Tokenizer::BLUE:      ctrl->SetWindowText("(1) Blue");    break;
			case Tokenizer::RED:       ctrl->SetWindowText("(2) Red");     break;
			case Tokenizer::GREEN:     ctrl->SetWindowText("(3) Green");   break;
			case Tokenizer::YELLOW:    ctrl->SetWindowText("(4) Yellow");  break;
			case Tokenizer::PURPLE:    ctrl->SetWindowText("(5) Purple");  break;
		}
	}

	else if(_controlType == Tokenizer::SEVEN_BUTTON){
		switch(_answer) {	
			case Tokenizer::NO_BUTTON:	ctrl->SetWindowText("<none>");	break;
			case Tokenizer::Y:			ctrl->SetWindowText("(1) Y");	break;
			case Tokenizer::X:			ctrl->SetWindowText("(2) X");	break;
			case Tokenizer::B:			ctrl->SetWindowText("(3) B");	break;
			case Tokenizer::A:			ctrl->SetWindowText("(4) A");	break;
			case Tokenizer::TOGGLE:			ctrl->SetWindowText("(5) Toggle");	break;
			case Tokenizer::LEFT_TRIGGER:	ctrl->SetWindowText("(6) Left Trigger"); break;
			case Tokenizer::RIGHT_TRIGGER:	ctrl->SetWindowText("(7) Right Trigger"); break;
			case Tokenizer::MOUSE:		ctrl->SetWindowText("Mouse Click");	break;
			default:					 ctrl->SetWindowText("<none>");		break;
		}
	}
}

void CEditTrialDialog::OnBnClickedCancel()
{
	// TODO: Add your control notification handler code here
	OnCancel();
}
