/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <sstream>
#include "TrialId.h"

TrialId::TrialId()
  : _type(TrialId::FILLER), _condition(0), _item(0), _index(0), _isSequence(false)
{}

TrialId::TrialId(Type type, unsigned short cond, unsigned short item, 
				 unsigned short index, bool seq)
  : _type(type), _condition(cond), _item(item), _index(index), _isSequence(seq)
{}

TrialId::TrialId(const std::string &identifier) throw(CExperimentException)
  : _type(TrialId::FILLER),_condition(0), _item(0), _index(0), _isSequence(false)
{
	std::istringstream is(identifier);
	std::ostringstream os;
	char ch;

	os<<"Invalid trial identifier '"<<identifier<<"':"<<std::endl;

	ch = is.get();
	if(ch == 'S' || ch == 's') {
		_isSequence = true;
		ch = is.get();
	}

	switch(ch) {
		case 'e':
		case 'E': _type = TrialId::EXPERIMENTAL;  break;
		case 'f':
		case 'F': _type = TrialId::FILLER;        break;
		case 'p':
		case 'P': _type = TrialId::PRACTICE;      break;
		default:  os<<"Unrecognized trial type."<<std::endl;
			      throw(CExperimentException(os.str().c_str()));
	}

	is>>_condition;
	if(is.fail()) {
		os<<"Error reading condition number."<<std::endl;
		throw(CExperimentException(os.str().c_str()));
	}

	ch = is.get();
	if((ch != 'I' && ch != 'i') || (is>>_item).fail()) {
		os<<"Error reading item number."<<std::endl;
		throw(CExperimentException(os.str().c_str()));
	}

	if(!_isSequence) {
		ch = is.get();
		if((ch != 'D' && ch != 'd') || (is>>_index).fail()) {
			os<<"Error reading dependent number."<<std::endl;
			throw(CExperimentException(os.str().c_str()));
		}
	}
}

TrialId::TrialId(const TrialId &id)
{
	_type       = id._type;
	_condition  = id._condition;
	_item       = id._item;
	_index      = id._index;
	_isSequence = id._isSequence;
}

TrialId::~TrialId()
{}

int TrialId::compare(const TrialId &id) const
{
	if(_type < id._type)
		return(-1);
	else if(_type > id._type)
		return(1);

	if(_item < id._item) 
		return(-1);
	if(_item == id._item && _condition < id._condition) 
		return(-1);
	if(_item == id._item && _condition == id._condition && _index < id._index)
		return(-1);
	if(_item == id._item && _condition == id._condition && _index == id._index)
		return(0);
	return(1);
}

unsigned short TrialId::condition() const
{
	return(_condition);
}

std::string TrialId::idString() const
{
	std::ostringstream os;

	if(_isSequence)
		os<<'S';

	switch(_type) {
		case TrialId::EXPERIMENTAL: os<<'E';  break;
		case TrialId::FILLER:       os<<'F';  break;
		case TrialId::PRACTICE:     os<<'P';  break;
	}
	
	os<<_condition<<'I'<<_item;
	if(!_isSequence)
		os<<'D'<<_index;
	return(os.str());
}

unsigned short TrialId::index() const
{
	return(_index);
}

bool TrialId::isDependent() const
{
	return(_index > 0);
}

bool TrialId::isExperimental() const
{
	return(_type == TrialId::EXPERIMENTAL);
}

bool TrialId::isFiller() const
{
	return(_type == TrialId::FILLER);
}

bool TrialId::isPractice() const
{
	return(_type == TrialId::PRACTICE);
}

bool TrialId::isSequence() const
{
	return(_isSequence);
}

unsigned short TrialId::item() const
{
	return(_item);
}

const TrialId &TrialId::operator=(const TrialId &rhs)
{
	_type       = rhs._type;
	_condition  = rhs._condition;
	_item       = rhs._item;
	_index      = rhs._index;
	_isSequence = rhs._isSequence;
	return(*this);
}

TrialId::Type TrialId::type() const
{
	return(_type);
}