/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "wavefile.hpp"

CWaveFile::CWaveFile(const std::string &fileName, bool openForRead) throw(CWaveException)
  : _nSamples(0), _sampleSize(0), _isReading(openForRead)
{
	if(openForRead)
		_waveFile.open(fileName.c_str(), std::ios::in | std::ios::binary);
	else 
		_waveFile.open(fileName.c_str(), std::ios::out | std::ios::binary);

	if(!_waveFile.is_open())
		throw(CWaveException("Cannot open wave file."));
}

CWaveFile::~CWaveFile()
{
	close();
}

void CWaveFile::close()
{
	if(!_isReading && _waveFile.is_open()) {
		DWORD dataSize = _nSamples * _sampleSize;
		ByteMap byteMap;

		_waveFile.seekp(4, std::ios::beg);       //write file size (minus 8)
		byteMap.dword = 36 + dataSize;
		_waveFile.write((char*)(byteMap.bytes), 4);

		_waveFile.seekp(40, std::ios::beg);      //write size of data block
		byteMap.dword = dataSize;
		_waveFile.write((char*)(byteMap.bytes), 4);
	}

	_waveFile.close();
}

DWORD CWaveFile::nSamples() const
{
	return(_nSamples);
}

void CWaveFile::readHeader(WAVEFORMATEX *format) throw(CWaveException)
{
	ByteMap byteMap;

	_waveFile.read((char*)(byteMap.bytes), 4);   //read "RIFF" marker
	if(byteMap.dword != 0x46464952) 
		throw(CWaveException("Error reading wave audio file: \nMissing 'RIFF' chunk."));

	_waveFile.read((char*)(byteMap.bytes), 4);   //read file length (discarded)

	_waveFile.read((char*)(byteMap.bytes), 4);   //read "WAVE" marker
	if(byteMap.dword != 0x45564157) 
		throw(CWaveException("Error reading wave audio file: \nMissing 'WAVE' marker."));

	_waveFile.read((char*)(byteMap.bytes), 4);   //read "fmt " marker
	if(byteMap.dword != 0x20746d66) 
		throw(CWaveException("Error reading wave audio file: \nMissing format chunk."));

	_waveFile.read((char*)(byteMap.bytes), 4);   //read format chunk size
	if(byteMap.dword != 16) 
		throw(CWaveException("Error reading wave audio file: \nUnexpected format chunk size."));

	byteMap.dword = 0;
	_waveFile.read((char*)(byteMap.bytes), 2);   //read format type
	format->wFormatTag = byteMap.word;
	if(byteMap.dword != 1) 
		throw(CWaveException("Error reading wave audio file: \nUnrecognized (non-PCM) format."));

	_waveFile.read((char*)(byteMap.bytes), 2);   //read number channels
	format->nChannels = byteMap.word;

	_waveFile.read((char*)(byteMap.bytes), 4);   //read number samples per second
	format->nSamplesPerSec = byteMap.dword;

	_waveFile.read((char*)(byteMap.bytes), 4);   //read number bytes per second
	format->nAvgBytesPerSec = byteMap.dword;

	byteMap.dword = 0;
	_waveFile.read((char*)(byteMap.bytes), 2);   //read block alignment (bytes per sample)
	_sampleSize = format->nBlockAlign = byteMap.word;

	_waveFile.read((char*)(byteMap.bytes), 2);   //read number bits per sample
	format->wBitsPerSample = byteMap.word;

	_waveFile.read((char*)(byteMap.bytes), 4);   //read "data" marker
	if(byteMap.dword != 0x61746164) 
		throw(CWaveException("Error reading wave audio file: \nMissing 'data' chunk."));

	_waveFile.read((char*)(byteMap.bytes), 4);   //read size of data block
	_nSamples = byteMap.dword / _sampleSize;
}

void CWaveFile::readWaveData(void *buf, int nBytes) throw(CWaveException)
{
	_waveFile.read(static_cast<char*>(buf), nBytes);

	if(_waveFile.eof())
		throw(CWaveException("Encountered unexpected EOF while reading wave audio file."));
}

void CWaveFile::reset()
{
	_nSamples = _sampleSize = 0;
	if(_isReading) _waveFile.seekg(0, std::ios::beg);
	else _waveFile.seekp(0, std::ios::beg);
}

void CWaveFile::writeHeader(const WAVEFORMATEX *format)
{
	ByteMap byteMap;

	_waveFile<<"RIFFxxxxWAVEfmt ";               //the "xxxx" acts as a place holder for later

	byteMap.dword = 16;                          //size of formatting chunk
	_waveFile.write((char*)(byteMap.bytes), 4);

	byteMap.dword = format->wFormatTag;          //formatting type (PCM)
	_waveFile.write((char*)(byteMap.bytes), 2);

	byteMap.dword = format->nChannels;           //number of channels
	_waveFile.write((char*)(byteMap.bytes), 2);

	byteMap.dword = format->nSamplesPerSec;      //number of samples per second
	_waveFile.write((char*)(byteMap.bytes), 4);

	byteMap.dword = format->nAvgBytesPerSec;     //number of bytes per second
	_waveFile.write((char*)(byteMap.bytes), 4);

	byteMap.dword = format->nBlockAlign;         //block alignment (bytes per sample)
	_sampleSize = format->nBlockAlign;
	_waveFile.write((char*)(byteMap.bytes), 2);

	byteMap.dword = format->wBitsPerSample;      //bits per sample
	_waveFile.write((char*)(byteMap.bytes), 2);

	_waveFile<<"dataxxxx";
	_waveFile.flush();
}

void CWaveFile::writeWaveData(const void *buf, int nBytes)
{
	_nSamples += nBytes / _sampleSize;
	_waveFile.write(static_cast<const char*>(buf), nBytes);
}
