/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <algorithm>
#include <sstream>
#include <sys/stat.h>
#include "experiment.hpp"
#include "random.hpp"
#include "tokenizer.h"

Experiment::Experiment()
: _backColor(0x0), _foreColor(0xffffff), _nConditions(0), _dcThreshold(0),
_nExperiments(0), _nTrials(0), _isSaved(false), _filterMode(Tokenizer::FILTER_OFF),
_calibrationType(Tokenizer::HV9), _displayType(Tokenizer::LCD), _controlType(Tokenizer::UNKNOWN_CONTROLLER)
{
	for (int i=0; i<20; i++){
		_expConditions[i] = 0;
	}
}

Experiment::~Experiment()
{
	while(!_sequences.empty()) {                 //delete all sequences
		Sequence *s = _sequences.front();
		_sequences.pop_front();
		delete s;
	}

	while(!_trials.empty()) {                    //delete all trials
		Trial *t = _trials.front();
		_trials.pop_front();
		delete t;
	}

	while(!_trialTypes.empty()) {                //delete all trial types
		TrialType *t = _trialTypes.front();
		_trialTypes.pop_front();
		delete t;
	}
}

bool Experiment::addSequence(const Sequence &s)
{
	const TrialId &ident = s.identifier();
	SequenceIter i = _sequences.begin();

	_isSaved = false;                            //mark experiment as changed

	while(i != _sequences.end()) {               //check if already exists
		int comp = ident.compare((*i)->identifier());

		if(comp == 0) {                          //if exists, replace
			Sequence &temp = **i;
			temp = s;
			return(true);
		}
		else if(comp < 0) {                      //belongs here
			_sequences.insert(i, new Sequence(s));
			return(false);
		}
		else ++i;
	}

_sequences.push_back(new Sequence(s));       //otherwise place at end
	return(false);
}

bool Experiment::addTrial(const Trial &t)
{
	const TrialId &ident = t.identifier();
	TrialIter i = _trials.begin();

	_isSaved = false;                            //mark experiment as updated

	while(i != _trials.end()) {                  //check if already exists
		int comp = ident.compare((*i)->identifier());

		if(comp == 0) {                          //if exists, replace
			Trial &temp = **i;
			temp = t;
			return(true);
		}
		else if(comp < 0) {                      //belongs here
			_trials.insert(i, new Trial(t));
			++_nTrials;
			return(false);
		}
		else ++i;
	}

	_trials.push_back(new Trial(t));             //otherwise place at end
	++_nTrials;
	return(false);
}

bool Experiment::addTrialType(TrialType &t)
{
	const std::string &ident = t.identifier();
	TrialTypeIter i = _trialTypes.begin(); 

	_isSaved = false;                            //mark experiment as updated

	while(i != _trialTypes.end()) {              //insert in alphabetical order
		int comp = ident.compare((*i)->identifier());

		if((*i)->controlType() != Tokenizer::UNKNOWN_CONTROLLER)
			_controlType = (*i)->controlType();

		if(comp == 0) {                          //if exists, replace
			TrialType &temp = **i;
			temp = t;
			return(true);
		}
		else if(comp < 0) {                      //belongs here
			_trialTypes.insert(i, new TrialType(t));
			return(false);
		}
		else ++i;
	}

	_trialTypes.push_back(new TrialType(t)); //otherwise place at end
	if(_controlType == Tokenizer::UNKNOWN_CONTROLLER)
		_controlType = t.controlType();
	return(false);
}

COLORREF Experiment::backgroundColor() const
{
	return(_backColor);
}

int Experiment::countTrials(const IdentifierList &l)
{
	int n = 0;

	for(IdentifierIter i = l.begin(); i != l.end(); ++i) {
		if(i->isSequence()) {
			Sequence *s = getSequence(*i);
			n += s->length();
		}
		else ++n;
	}
	return(n);
}

void Experiment::deleteSequence(const std::string &identifier)
{
	for(SequenceIter i = _sequences.begin(); i != _sequences.end(); ++i) {
		const TrialId &id = (*i)->identifier();
		int comp = identifier.compare(id.idString());

		if(comp == 0) {
			delete (*i);
			_sequences.erase(i);
			_isSaved = false;
			return;
		}
	}
}

void Experiment::deleteTrial(const std::string &identifier)
{
	for(TrialIter i = _trials.begin(); i != _trials.end(); ++i) {
		const TrialId &id = (*i)->identifier();
		int comp = identifier.compare(id.idString());

		if(comp == 0) {
			delete (*i);
			_trials.erase(i);
			_isSaved = false;
			return;
		}
	}
}

void Experiment::deleteTrialType(const std::string &identifier)
{
	for(TrialTypeIter i = _trialTypes.begin(); i != _trialTypes.end(); ++i) {
		int comp = identifier.compare((*i)->identifier());

		if(comp == 0) {
			delete (*i);
			_trialTypes.erase(i);
			_isSaved = false;
			return;
		}
	}
}

COLORREF Experiment::foregroundColor() const
{
	return(_foreColor);
}

Sequence *Experiment::getSequence(const TrialId &identifier) const
{                                                //return specific sequence if present
	for(constSequenceIter i=_sequences.begin(); i != _sequences.end(); ++i) {
		if(identifier.compare((*i)->identifier()) == 0)
			return(*i);
	}
	return(0);
}

IdentifierList Experiment::getSequenceNames() const
{
	IdentifierList list;

	for(constSequenceIter i = _sequences.begin(); i != _sequences.end(); ++i) {
		const Sequence &s = **i;
		list.push_back(s.identifier().idString());
	}
	return(list);
}

Trial *Experiment::getTrial(const TrialId &identifier) /*const*/
{                                                //return specific trial if present
	for(constTrialIter i=_trials.begin(); i != _trials.end(); ++i) {
		if(identifier.compare((*i)->identifier()) == 0)
			return(*i);
	}
	return(0);
}

Trial *Experiment::getTrial(const TrialId &identifier) const
{                                                //return specific trial if present
	for(constTrialIter i=_trials.begin(); i != _trials.end(); ++i) {
		if(identifier.compare((*i)->identifier()) == 0)
			return(*i);
	}
	return(0);
}

TrialType *Experiment::getTrialType(const std::string &identifier) const
{                                                //return named type if present
	for(constTrialTypeIter i=_trialTypes.begin(); i != _trialTypes.end(); ++i)
		if(identifier.compare((*i)->identifier()) == 0)
			return(*i);
	return(0);
}

IdentifierList Experiment::getTrialNames(TrialId::Type type) const
{
	IdentifierList list;

	for(constTrialIter i = _trials.begin(); i != _trials.end(); ++i) {
		const TrialId &id = (*i)->identifier();
		
		switch(type) {
		case TrialId::DEPENDENT:	if(id.isDependent())
										list.push_back(id.idString());
									break;
		case TrialId::INDEPENDENT:	if(!id.isDependent())
										list.push_back(id.idString());
									break;
		case TrialId::EXPERIMENTAL:	if(id.isExperimental())
										list.push_back(id.idString());
									break;
		case TrialId::FILLER:		if(id.isFiller())
										list.push_back(id.idString());
									break;
		case TrialId::PRACTICE:		if(id.isPractice())
										list.push_back(id.idString());
									break;
		default:					list.push_back(id.idString());
		}
	}
	return(list);
}

StringList Experiment::getTrialTypeNames() const
{
	StringList list;

	for(constTrialTypeIter i = _trialTypes.begin(); i != _trialTypes.end(); ++i) {
		const TrialType &t = **i;
		list.push_back(t.identifier());
	}
	return(list);
}

bool Experiment::hasInlineText() const
{
	for(constTrialTypeIter i=_trialTypes.begin(); i != _trialTypes.end(); ++i)
		if((*i)->textFormat() != 0)
			return(true);
	return(false);
}

const std::string Experiment::header(bool formatForEditBox) const
{
	if(formatForEditBox) {
		std::string str(_header);
		std::string::size_type idx = str.find("\n");

		while(idx != std::string::npos) {        //replace '\n' with "\r\n"
			str.insert(idx, "\r");
			idx = str.find("\n", idx+2);
		}
		return(str);
	}
	
	return(_header);
}

bool Experiment::isSaved() const
{
	return(_isSaved);
}

void Experiment::load(const std::string &fileName) throw(CExperimentException)
{
	std::ostringstream os;
	Tokenizer tok;
	Tokenizer::Keyword keyword;
	int i=0;

	tok.open(fileName);

	_header = tok.readHeader();
	keyword = tok.readKeyword();
	while(tok.isOkay() && keyword == Tokenizer::SET) {
		char ch;

		keyword = tok.readKeyword();
		tok.readWhitespace();
		ch = tok.readChar();

		if(ch != '=') {
			os<<"Syntax error in global setting 'set "<<Tokenizer::decode(keyword);
			os<<"':\nExpected '=' sign, found '"<<ch<<"'."<<std::endl;
			throw(CExperimentException(os.str().c_str()));
		}

		switch(keyword) {
		case Tokenizer::BACKGROUND:
			_backColor = tok.readColor();
			break;
		case Tokenizer::CONDITIONS:
			_nConditions = tok.readInteger();
			break;
		case Tokenizer::EXPERIMENTS:
			_nExperiments = tok.readInteger();
			break;
		case Tokenizer::EXPCONDITIONS:
			//tok.readExpConditions(_expConditions);
			i=0;
			do{
				int  next = tok.readInteger();
				//if (next > 0 && next < _nExperiments)
					_expConditions[i++] = next;
				//else 
				//	_expConditions[i++] = 1;
			}while(tok.isOkay() && i<_nExperiments);
			_expConditions[i] = 0;
			if(_nExperiments < 20){
				for(int i=_nExperiments; i<20; i++)
					_expConditions[i] = 0;
			}
			break;
		case Tokenizer::FOREGROUND:
			_foreColor = tok.readColor();
			break;
		case Tokenizer::FILTERMODE:
			_filterMode = tok.readFilterMode();
			break;
		case Tokenizer::WINDOW_THRESHOLD:
			_dcThreshold = tok.readInteger();
			break;
		case Tokenizer::CALIBRATION_TYPE:
			_calibrationType = tok.readCalibrationType();
			break;
		case Tokenizer::CALIBRATION_HEIGHT:
			_calibrationHeight = tok.readInteger();
			break;
		case Tokenizer::DISPLAY_TYPE:
			_displayType = tok.readDisplayType();
			break;
		default:
			os<<"Error loading script file.\n\nUnrecognized global setting 'set ";
			if(keyword == Tokenizer::UNKNOWN)
				os<<tok.readString()<<"'."<<std::endl;
			else os<<Tokenizer::decode(keyword)<<"'."<<std::endl;
			throw(CExperimentException(os.str().c_str()));
		}

		keyword = tok.readKeyword();
	}

	if(_nConditions == 0)
		throw(CExperimentException("Number of experimental conditions not set in script."));

	if(_nExperiments == 0)
		throw(CExperimentException("Number of experiments in script not set."));

	while(tok.isOkay()) {
		if(keyword == Tokenizer::SEQUENCE) {
			Sequence s(tok);

			if(getSequence(s.identifier().idString())) {
				os<<"Error loading script file.\n";
				os<<"Multiply defined trial sequence: "<<s.identifier().idString()<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			addSequence(s);
		}
		else if(keyword == Tokenizer::TRIAL) {
			Trial t(tok);                        //generate trial
			std::string ident = t.name();

			if(getTrialType(t.typeId()) == 0) {  //error: cond defn not found
				os<<"Syntax error reading trial '"<<ident<<"'.\n\n";
				os<<"Undefined trial condition: "<<t.typeId()<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
	      
			if(getTrial(ident)) {                //error: trial already defined
				os<<"Error loading script file.\n\n";
				os<<"Multiply defined trial: "<<ident<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			t.setControlType(getTrialType(t.typeId())->controlType());

			addTrial(t);                         //ok: add trial to list
		}
		else if(keyword == Tokenizer::TRIALTYPE) {
			TrialType t(tok);

			if(getTrialType(t.identifier())) {   //error: cond already defined
				os<<"Error loading script file.\n\n";
				os<<"Multiply defined trial condition: "<<t.identifier()<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			addTrialType(t);                     //ok: add to list
			if(t.controlType() != Tokenizer::UNKNOWN_CONTROLLER)
				_controlType = t.controlType();
		}
		else {
			os<<"Unexpected keyword '"<<Tokenizer::decode(keyword)<<"' in script file.\n";
			os<<"Expected either Sequence, Trial or Trial Type definition."<<std::endl;
			throw(CExperimentException(os.str().c_str()));
		}

		keyword = tok.readKeyword();
	}
	_isSaved = true;                             //script matches file
}

int Experiment::nConditions() const
{
	return(_nConditions);
}

int Experiment::nExperiments() const
{
	return(_nExperiments);
}

int Experiment::nTrials() const
{
	return(_nTrials);
}

Tokenizer::FilterMode Experiment::filterMode() const
{
	return(_filterMode);
}


void Experiment::randomize(IdentifierList &l, Random &rng) const
{
	for(int i = static_cast<int>(l.size()); i > 0; --i) {
		IdentifierList::iterator ti = l.begin();
		int idx = rng.randomInt(0, i);

		for(int j = 0; j < idx; ++j, ++ti);

        l.push_back(*ti);
		l.erase(ti);
	}
}

void Experiment::save(std::ostream &os)
{
	os<<"%BeginHeader\n"<<_header
	  <<"%EndHeader\n"<<std::endl;

	os<<"set conditions =  "<<_nConditions<<std::endl;
	os<<"set experiments = "<<_nExperiments<<std::endl;
	os<<"set expConditions =";
	for (int i=0; i<_nExperiments; i++)
		os<<" "<<_expConditions[i];
	os<<std::endl;
	os<<"set background =  "<<_backColor<<std::endl;
	os<<"set foreground =  "<<_foreColor<<std::endl;
	os<<"set filterMode = "<<_filterMode<<std::endl;
	os<<"set windowThreshold = "<<_dcThreshold<<std::endl;
	os<<"set calibration_type = "<<_calibrationType<<std::endl;
	if(_calibrationHeight != 0){
		os<<"set calibration_height = "<<_calibrationHeight<<std::endl;
	}
	os<<"set display_type = "<<Tokenizer::decode(_displayType)<<'\n'<<std::endl;

	for(TrialTypeIter i = _trialTypes.begin(); i != _trialTypes.end(); ++i)
		(*i)->save(os);

	for(TrialIter i = _trials.begin(); i != _trials.end(); ++i)
		(*i)->save(os);

	for(SequenceIter i = _sequences.begin(); i != _sequences.end(); ++i)
		(*i)->save(os);

	_isSaved = true;
}

IdentifierList Experiment::selectTrials(const std::string &fileName) const 
	throw(CExperimentException)
{                                                //use trial order given in file
	IdentifierList list;
	Tokenizer tok;
	std::string name;

	tok.open(fileName);
	tok.readHeader();

	name = tok.readString();
	while(tok.isOkay()) {
		TrialId id(name);

		if((!id.isSequence() && getTrial(id)) ||
		   (id.isSequence() && getSequence(id)))
			list.push_back(id);
		else {
			std::ostringstream os;
			os<<"Error loading script:\nUnrecognized trial or sequence ID '"<<name<<"'"<<std::endl;
			throw(CExperimentException(os.str().c_str()));
		}

		name = tok.readString();
	}
	return(list);
}

IdentifierList Experiment::selectTrials(int expId, int groupID, int itemNum) const
{                                                //use trials as they appear in the script
	IdentifierList list;
	const int maxSubExpts = 20;

	for(constTrialIter i = _trials.begin(); i != _trials.end(); ++i) {
		const TrialId &id = (*i)->identifier();
		Trial *t = 0;

		bool included=false;
		int expNum;
		// ORIGINAL
		if(_expConditions[0] == 0){
			included = ((id.item()+groupID-1)%_nConditions) == (id.condition()%_nConditions);
			expNum = ((id.condition()-1) / _nConditions)+1;
		}
		// end orig

		//int _nCond[3] = {2,3,2};
		//int starts[3] = {1,3,6};
		//int ends[3] = {2,5,7};
		else{
		int starts[maxSubExpts], ends[maxSubExpts];

		starts[0] = 1;
		for (int k=1; k<maxSubExpts; k++){
			if(k<_nExperiments)
				starts[k] = starts[k-1] + _expConditions[k-1];
			else starts[k] = 0;
		}

		for (int k=0; k<_nExperiments && k<maxSubExpts; k++){
			if(k<_nExperiments-1)
				ends[k] = starts[k+1]-1;
		}
		ends[_nExperiments-1] = _nConditions;

		expNum=1;
		int j=0;
		while(j++ < nExperiments()){
			if (id.condition() >= starts[j-1]){
				if(j < nExperiments()){
					if(id.condition() < starts[j]){
						expNum = j;
						break;
					}
				}
				else expNum = j;
			}
		}

		//bool included = ((id.item()+groupID-1)%_expConditions[expNum-1]) == ( ((id.condition()-1) % _expConditions[expNum-1])-1);
		
		if(expNum>1)
			included = (((id.item() + groupID-1 ))%(_expConditions[expNum-1])) == ((id.condition()-ends[expNum-2])%_expConditions[expNum-1]);
	
		else 
			included = ((id.item() + groupID-1)%(_expConditions[expNum-1])) == (id.condition()%_expConditions[expNum-1]);
		/*int t1,t2;
		if (expNum <=1){
				t1 = (id.item() + groupID-1);
				t2 = (id.condition());
		}
		else{
			t1 = (id.item() + groupID-1)%(ends[expNum-2]);
			t2 = id.condition()%(ends[expNum-2]);
		}

		
		bool included = t1 == t2;*/
		} //end check included

		if(id.isDependent())                     //skip all dependents
			continue;

		if(id.isFiller() && (expId == EXPT_ALL || expId == EXPT_FILLER))
			t = *i;
		if(id.isPractice() && (expId == EXPT_ALL || expId == EXPT_PRACTICE))
			t = *i;
		if(id.isExperimental() && included && (expId == expNum || expId == EXPT_ALL)) 
			t = *i;

		if(t != 0) {                             //if this trial is selected
			Sequence *s = getSequence(id);      //  check for a sequence
			if(s) list.push_back(s->identifier());
			else list.push_back(id);
		}
	}
	if (itemNum){
		IdentifierList head;

		while(!list.empty() && (list.back().condition() != groupID || list.back().item() != itemNum) ){
				int cond = list.back().condition();
				int it = list.back().item();	
				head.push_front(list.back());
				list.pop_back();
		}
		if(!list.empty() && list.back().condition() == groupID && list.back().item() == itemNum){
			int cond = list.back().condition();
			int it = list.back().item();
			head.push_front(list.back());
			list.pop_back();
		}
		while(!list.empty()){
			int cond = list.front().condition();
			int it = list.front().item();
			head.push_back(list.front());
			list.pop_front();
		}
		
		return(head);
	}	
	return(list);
}

IdentifierList Experiment::selectTrials(int groupID, bool interleave) const
	throw(CExperimentException)
{
	Random rng;
	const int maxSubExpts = 20;
	IdentifierList expt[maxSubExpts], filler, list;
	unsigned int nExpt = 0, nFiller, nPractice;

	if(_nExperiments >= maxSubExpts)
		throw(CExperimentException("Too many subexperiments: maximum is 20."));

	for(int i=0; i < _nExperiments; ++i) {       //get list of trials for each subexperiment
		expt[i] = selectTrials(i+1, groupID);
		nExpt += static_cast<unsigned int>(expt[i].size());
		randomize(expt[i], rng);
	}

	filler = selectTrials(EXPT_FILLER, groupID); //get list of filler trials
	nFiller = static_cast<unsigned int>(filler.size());
	randomize(filler, rng);

	list = selectTrials(EXPT_PRACTICE, groupID); //init list with practice trials
	nPractice = static_cast<unsigned int>(list.size());
	randomize(list, rng);

	if(interleave && _nExperiments > 1) {        //interleave experiment trials
		int last = -1, len, permute[maxSubExpts];

		for(int i=0; i < _nExperiments; ++i)     //initialize permutation order
			permute[i] = i;

		for(unsigned int i=0; i < nExpt; ++i) {  //randomly select expt trials
			unsigned int j = i % _nExperiments;

			if(j == 0) {                         //choose a new permutation of subexperiments
				do {
					for(int k=0; k < _nExperiments; ++k) {
						int idx1 = rng.randomInt(0, _nExperiments);
						int idx2 = rng.randomInt(0, _nExperiments);
						int tmp = permute[idx1];
						permute[idx1] = permute[idx2];
						permute[idx2] = tmp;
					}
				} while(permute[0] == last);
				last = permute[_nExperiments-1];
			}

			list.push_back(expt[permute[j]].front());
			expt[permute[j]].pop_front();
		}

		len = 2*nExpt + nPractice;
		while(!filler.empty()) {                 //randomly place the fillers (after practice)
			IdentifierList::iterator i = list.begin();
			int r = rng.randomInt(nPractice+1, len);

			while(--r >= 0) ++i;
			list.insert(i, filler.front());
			filler.pop_front();
			++len;
		}
	}
	else {                                       //no interleaving, randomly place trials on list
		for(int i=1; i < _nExperiments; ++i)     //merge all sub-experiments
			expt[0].splice(expt[0].end(), expt[i]);
		expt[0].splice(expt[0].end(), filler);   //merge filler and experiments
		randomize(expt[0], rng);                 //randomize merged list
		list.splice(list.end(), expt[0]);        //merge with practice list
	}

	return(list);
}

void Experiment::setColors(COLORREF bkgd, COLORREF fgd)
{
	_backColor = bkgd;
	_foreColor = fgd;
	_isSaved = false;
}

void Experiment::setConditions(int nConds, int nExp)
{
	_nConditions = nConds;
	_nExperiments = nExp;
	_isSaved = false;
}

void Experiment::setHeader(const std::string &header)
{
	_header = header;
	std::string::size_type idx = _header.find("\r\n");

	while(idx != std::string::npos) {            //replace "\r\n" with just '\n'
		_header.replace(idx, 1, "\n");
		_header.erase(idx, 1);
		idx = _header.find("\r\n", idx);
	}

	if(_header[_header.size()-1] != '\n')        //make sure header ends with newline
		_header += '\n';
	_isSaved = false;
}

void Experiment::setFilterMode(const Tokenizer::FilterMode fMode)
{
	_filterMode = fMode;
}

void Experiment::setDisplayType(const Tokenizer::DisplayType dType)
{
	_displayType = dType;
}

void Experiment::setDCThreshold(unsigned int dcThresh)
{
	_dcThreshold = dcThresh;
}

Tokenizer::FilterMode Experiment::getFilterMode()
{
	return _filterMode;
}

unsigned int Experiment::getDCThreshold()
{
	return _dcThreshold;
}

Tokenizer::DisplayType Experiment::getDisplayType()
{
	return _displayType;
}

Tokenizer::CalibrationType Experiment::calibrationType()
{
	return _calibrationType;
}

int Experiment::calibrationHeight()
{
	return _calibrationHeight;
}
void Experiment::setCalibrationType(const Tokenizer::CalibrationType cType)
{
	_calibrationType = cType;
}

bool Experiment::validate(std::ostream &os) const
{
	bool isValid = true;

	if(_nConditions == 0)
		os<<" * Number of experiment conditions must be set (currently zero)."<<std::endl;
	if(_nExperiments == 0)
		os<<" * Number of experiments must be set (currently zero)."<<std::endl;

	for(constTrialIter i = _trials.begin(); i != _trials.end(); ++i) {
		Trial *t = *i;
		TrialType *tc = getTrialType(t->typeId());
		struct _stat fStatus;

		if(!tc) {                                //check trial type
			isValid = false;
			os<<" * Trial '"<<t->name()<<"' has invalid trial type '";
			os<<t->typeId()<<"'."<<std::endl;
		}
                                                 //check playback file
		if(t->hasAudio() && _stat(t->audioData().path.c_str(), &fStatus) < 0) {
			isValid = false;
			os<<" * Trial '"<<t->name()<<"' has invalid wave audio file '";
			os<<t->audioData().path<<"'."<<std::endl;
		}
                                                 //check for text properties
		if(t->isTextDisplay() && tc->textFormat() == 0) {
			isValid = false;
			os<<" * Trial '"<<t->name()<<"' has inline text, but trial type '";
			os<<t->typeId()<<"' does not have text enabled."<<std::endl;
		}
                                                 //check image files
		if(!t->isTextDisplay() && _stat(t->display1().c_str(), &fStatus) < 0) {
			isValid = false;
			os<<" * Trial '"<<t->name()<<"' has invalid image file '";
			os<<t->display1()<<"'."<<std::endl;
		}

		if(tc->nButtons() <= 0 && !t->hasAudio() && t->maxDisplayTime() == 0) {
			isValid = false;
			os<<" * Trial '"<<t->name()<<"' has trial type '"<<t->typeId();
			os<<"'\nwith zero buttons defined, but no audio playback and\n";
			os<<"maximum display time of zero."<<std::endl;
		}

		if(t->questionAnswer() != Tokenizer::NO_BUTTON) {
			bool ok = false;

			for(int i=0; i < tc->nButtons(); ++i) {
				if(t->questionAnswer() == (tc->buttons())[i]) {
					ok = true;
					break;
				}
			}
			
			if(!ok) {
				isValid = false;
				os<<" * Trial '"<<t->name()<<"' is a question, but trial type '";
				os<<t->typeId()<<"' does not allow the subject to select the answer button."<<std::endl;
			}
		}
	}

	for(constSequenceIter i = _sequences.begin(); i != _sequences.end(); ++i) {
		Sequence &s = **i;
		
		s.resetTrialPtr();
		while(!s.atEnd()) {
			const TrialId &id = s.nextTrialId();
			if(getTrial(id) == 0) {
				isValid = false;
				os<<" * Sequence '"<<s.identifier().idString()<<"' contains invalid trial '";
				os<<id.idString()<<"'."<<std::endl;
			}
		}
	}

	return(isValid);
}

Tokenizer::ControlType Experiment::getControlType() const {
	return _controlType;
}

void Experiment::setExpConditions(int ec[20]) {
	//for (int i=0; i<nExperiments(); i++){
	for(int i=0; i<20; i++){
		_expConditions[i] = ec[i];
	}
	return;
}

void Experiment::getExpConditions(int ec[20]){
	//for (int i=0; i<nExperiments(); i++){
	for(int i=0; i<20; i++){
		ec[i] = _expConditions[i];
	}
	return;
}