/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"                              //needed for font rendering values
#include "trial-type.hpp"
#include "w32_exptsppt2.h"                       //needed for button values

#include <sstream>

/********************************************************************************************/

TrialType::TrialType(const TrialType &tc)
  : _audioFmt(0), _textFmt(0), _button(0), _textFmt2(0)
{
	(*this) = tc;
}

TrialType::TrialType(Tokenizer &tok) throw(CExperimentException)
  : _audioFmt(0), _textFmt(0), _button(0), _output(Tokenizer::NO_OUTPUT), 
  _trigger(Tokenizer::NO_TRIGGER), _controlType(Tokenizer::UNKNOWN_CONTROLLER), _cursorSize(0), _dcDelay(0), _nButtons(0), _interCharSpacing(100),
	_stimulusDelay(0), _revertDisplay(false), _textFmt2(0), _hColor(0)
{
	std::ostringstream os;
	std::string sTmp;
	Tokenizer::Keyword keyword;
	Tokenizer::ButtonType bTmp[10];
//	Tokenizer::ButtonTypeSideWinder bTmp7[10];

	_identifier = tok.readLine();              //read condition identifier
	os<<"Syntax error in definition of type "<<_identifier<<":\n";

	do {                                         //read fields (order independent)
		char ch;

		keyword = tok.readKeyword();             //read the field identifier
		if(keyword != Tokenizer::END) {
			ch = tok.readChar();                 //  read the '=' sign
			if(ch != '=') {                      //  error if no '=' sign
				os<<"  Field '"<<Tokenizer::decode(keyword);
				os<<"' expected '=', found '"<<ch<<"'"<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
		}
		Tokenizer::ControlType c = _controlType;
		switch(keyword) {                        //read the field data
		case Tokenizer::AUDIO_FORMAT:
			if(_audioFmt == 0) 
				_audioFmt = new AudioCaptureData;
			_audioFmt->length = tok.readInteger();
			_audioFmt->nChannels = tok.readAudioChannels();
			_audioFmt->sampleRate = tok.readInteger();
			_audioFmt->sampleSize = tok.readInteger();
			_audioFmt->path = tok.readString();
			break;
		case Tokenizer::BUTTON:
			//Tokenizer::ControlType c = _controlType;
			bTmp[_nButtons++] = tok.readButton(&c);
			if (_controlType == Tokenizer::UNKNOWN_CONTROLLER)
				_controlType = c;
			//_controlType = Tokenizer::FIVE_BUTTON;
			break;							   
		case Tokenizer::END:
			sTmp = tok.readLine();
			break;
		case Tokenizer::CURSOR_SIZE:
			_cursorSize = tok.readInteger();
			break;
		case Tokenizer::DC_DELAY:
			_dcDelay = tok.readInteger();
			break;
		case Tokenizer::OUTPUT:
			_output = tok.readOutput();
			break;
		case Tokenizer::INTER_CHAR:
			_interCharSpacing = tok.readInteger();
			_textFmt->spacingDiff = _interCharSpacing;
			break;
		case Tokenizer::REVERT:
			if(tok.readInteger() == 1)
				_revertDisplay = true;
			break;
		case Tokenizer::STIMULUS_DELAY:
			_stimulusDelay = tok.readInteger();
			break;
		case Tokenizer::TEXT_WEIGHT:
			if(_textFmt2 != 0){
				_textFmt2->fontWt = tok.readTextWeight();
				_textFmt2->italic = tok.readTextItalic();
			}
			else{
				if(_textFmt == 0)
					_textFmt = new TextFormat;
				_textFmt->fontWt = tok.readTextWeight();
				_textFmt->italic = tok.readTextItalic();
			}
			break;
		case Tokenizer::HIGHLIGHT_COLOR:
			//_hColor = new COLORREF;
			_hColor = tok.readColor();
			break;
		case Tokenizer::TEXT_FORMAT:
			if(_textFmt == 0) {
				_textFmt = new TextFormat;
				_textFmt->fontName = tok.readString();
				_textFmt->fontHt = tok.readInteger();
				_textFmt->lineSpace = tok.readInteger();
				_textFmt->xOffset = tok.readInteger();
				_textFmt->yOffset = tok.readInteger();
				_textFmt->rendering = tok.readTextQuality();
				_textFmt->italic = FALSE;
				_textFmt->fontWt = FW_NORMAL;
			}
			else {
				_textFmt2 = new TextFormat;
				_textFmt2->fontName = tok.readString();
				_textFmt2->fontHt = tok.readInteger();
				_textFmt2->lineSpace = tok.readInteger();
				_textFmt2->xOffset = tok.readInteger();
				_textFmt2->yOffset = tok.readInteger();
				_textFmt2->rendering = tok.readTextQuality();
				_textFmt2->italic = FALSE;
				_textFmt2->fontWt = FW_NORMAL;
			}

			break;
		case Tokenizer::TRIGGER:
			_trigger = tok.readTrigger();
			break;
		default:
			os<<"  Unrecognized field type."<<std::endl;
			throw(CExperimentException(os.str().c_str()));
		}
	} while(tok.isOkay() && keyword != Tokenizer::END);

	if(!tok.isOkay()) {                          //test for read failure
		os<<"  Unrecognized argument to "<<Tokenizer::decode(keyword)<<" field."<<std::endl;
		throw(CExperimentException(os.str().c_str()));
	}

	if(_identifier.compare(sTmp) != 0) {         //make sure "end" antches "define"
		os<<"  Mismatched 'end' marker: "<<sTmp<<std::endl; 
		throw(CExperimentException(os.str().c_str()));
	}

	if(_nButtons > 0) {
		_button = new Tokenizer::ButtonType[_nButtons];
		for(int i=0; i < _nButtons; ++i) {
			_button[i] = bTmp[i];
			if(bTmp[i] == Tokenizer::MOUSE && _nButtons > 1) {
				os<<"Mouse enabled in conjunction with other subject response buttons."<<std::endl;
				delete _button;
				throw(CExperimentException(os.str().c_str()));
			}
		}
	}
}

TrialType::TrialType(const std::string &ident, Tokenizer::ButtonType btn[],
					 Tokenizer::OutputType out, Tokenizer::TriggerType trig,
					 int cursor, int dcDelay, int stimDelay, int nButtons,
					 bool revertDisplay, Tokenizer::ControlType ctrl)
  : _identifier(ident), _audioFmt(0), _textFmt(0), _button(0), 
	_output(out), _trigger(trig), _cursorSize(cursor), _nButtons(nButtons), 
	_dcDelay(dcDelay), _stimulusDelay(stimDelay), _revertDisplay(revertDisplay), _controlType(ctrl), _textFmt2(0)
{
	if(_nButtons > 0) {
		_button = new Tokenizer::ButtonType[_nButtons];
		for(int i=0; i < _nButtons; ++i)
			_button[i] = btn[i];
	}
}

TrialType::~TrialType()
{
	if(_audioFmt) delete _audioFmt;
	if(_textFmt) delete _textFmt;
	if(_button) delete _button;
}

COLORREF TrialType::getHColor() const {
	return(_hColor);
}

const AudioCaptureData *TrialType::audioFormat() const
{
	return(_audioFmt);
}

Tokenizer::ButtonType *TrialType::buttons() const
{
	return(_button);
}

Tokenizer::ControlType TrialType::controlType() const
{
	return(_controlType);
}

int TrialType::cursorSize() const
{
	return(_cursorSize);
}

int TrialType::displayChangeDelay() const
{
	return(_dcDelay);
}

bool TrialType::doButtonResponse() const
{
	return(_nButtons > 0 && _button[0] != Tokenizer::MOUSE);
}

bool TrialType::doDelay() const
{
	return(_trigger == Tokenizer::NOGAZE);
}

bool TrialType::doDisplayRevert() const
{
	return(_revertDisplay);
}

bool TrialType::doDriftCorrection() const
{
	return(_trigger == Tokenizer::DRIFTCORRECT || _trigger == Tokenizer::DRIFTANDGAZE);
}

bool TrialType::doGazeContingentStart() const
{
	return(_trigger == Tokenizer::GAZE || _trigger == Tokenizer::DRIFTANDGAZE);
}

bool TrialType::doMouseResponse() const
{
	return(_nButtons > 0 && _button[0] == Tokenizer::MOUSE);
}

bool TrialType::doStreamOutput() const
{
	return(_output == Tokenizer::STREAM);
}

const std::string &TrialType::identifier() const
{
	return(_identifier);
}

int TrialType::nButtons() const
{
	return(_nButtons);
}

const TrialType &TrialType::operator=(const TrialType &tc)
{
	_identifier = tc._identifier;
	_output = tc._output;
	_trigger = tc._trigger;
	_cursorSize = tc._cursorSize;
	_dcDelay = tc._dcDelay;
	_nButtons = tc._nButtons;
	_stimulusDelay = tc._stimulusDelay;
	_revertDisplay = tc._revertDisplay;
	_controlType = tc._controlType;
	_interCharSpacing = tc._interCharSpacing;
	_hColor = tc._hColor;

	if(tc._audioFmt) {                           //copy audio format
		if(_audioFmt == 0) _audioFmt = new AudioCaptureData;
		(*_audioFmt) = *(tc._audioFmt);
	}
	else if(_audioFmt) {
		delete _audioFmt;
		_audioFmt = 0;
	}

	if(tc._textFmt) {                            //copy text format
		if(_textFmt == 0) _textFmt = new TextFormat;
		(*_textFmt) = *(tc._textFmt);
	}
	else if(_textFmt) {
		delete _textFmt;
		_textFmt = 0;
	}

	if(tc._textFmt2) {                            //copy text format
		if(_textFmt2 == 0) _textFmt2 = new TextFormat;
		(*_textFmt2) = *(tc._textFmt2);
	}
	else if(_textFmt2) {
		delete _textFmt2;
		_textFmt2 = 0;
	}

	if(_button) delete _button;
	if(_nButtons > 0) {
		_button = new Tokenizer::ButtonType[_nButtons];
		for(int i=0; i < _nButtons; ++i)
			_button[i] = tc._button[i];
	}
	else _button = 0;

	return(*this);
}

Tokenizer::OutputType TrialType::output() const
{
	return(_output);
}

void TrialType::save(std::ostream &os) const
{
	os<<"trial_type "<<_identifier<<std::endl;

	if(_audioFmt) {
		os<<"  audio_format =   "<<_audioFmt->length<<' ';
		if(_audioFmt->nChannels == 1) os<<" mono ";
		else os<<" stereo ";
		os<<_audioFmt->sampleRate<<' ';
		os<<_audioFmt->sampleSize<<" '";
		os<<_audioFmt->path<<'\''<<std::endl;
	}

	if(_textFmt) {
		os<<"  text_format =    '"<<_textFmt->fontName<<"' ";
		os<<_textFmt->fontHt<<' ';
		os<<_textFmt->lineSpace<<' ';
		os<<_textFmt->xOffset<<' ';
		os<<_textFmt->yOffset<<' ';
		os<<Tokenizer::decode(_textFmt->rendering)<<std::endl;
		
		os<<"  text_weight =    ";
		os<<Tokenizer::decodeWeight(_textFmt->fontWt)<<' ';
		os<<Tokenizer::decodeItalic(_textFmt->italic)<<std::endl;
	}	

	if(_textFmt2) {
		os<<"  text_format =    '"<<_textFmt2->fontName<<"' ";
		os<<_textFmt2->fontHt<<' ';
		os<<_textFmt2->lineSpace<<' ';
		os<<_textFmt2->xOffset<<' ';
		os<<_textFmt2->yOffset<<' ';
		os<<Tokenizer::decode(_textFmt2->rendering)<<std::endl;
		
		os<<"  text_weight =    ";
		os<<Tokenizer::decodeWeight(_textFmt2->fontWt)<<' ';
		os<<Tokenizer::decodeItalic(_textFmt2->italic)<<std::endl;
	}	

	for(int i=0; i < _nButtons; ++i) {            //write buttons
		os<<"  button =         "; 
		if (_controlType == Tokenizer::FIVE_BUTTON)
			os<<Tokenizer::decode5(_button[i])<<std::endl;
		else 
			os<<Tokenizer::decode7(_button[i])<<std::endl;
	}
	os<<"  output =         "<<Tokenizer::decode(_output)<<std::endl;
	os<<"  trigger =        "<<Tokenizer::decode(_trigger)<<std::endl;
	os<<"  cursor_size =    "<<_cursorSize<<std::endl;
	os<<"  dc_delay =       "<<_dcDelay<<std::endl;
	os<<"  stimulus_delay = "<<_stimulusDelay<<std::endl;
	os<<"  revert =         "<<_revertDisplay<<std::endl;
	if(_textFmt->spacingDiff != 100){
		os<<"  inter_char =     "<<_textFmt->spacingDiff<<std::endl;
	}
	//if(_hColor){
		os<<"  highlight_color ="<<_hColor<<std::endl;
	//}
	os<<"end "<<_identifier<<'\n'<<std::endl;
}

void TrialType::setAudioFormat(const AudioCaptureData &format)
{
	if(_audioFmt == 0) _audioFmt = new AudioCaptureData;
	(*_audioFmt) = format;
}

void TrialType::setTextFormat(const TextFormat &format, bool set2 )
{
	if(_textFmt == 0) _textFmt = new TextFormat;
	if(set2){
		(*_textFmt2) = format;
	}
	else{
		(*_textFmt) = format;
	}
}

int TrialType::getInterCharSpacing() const {
	return _interCharSpacing;
}

int TrialType::stimulusDelay() const
{
	return(_stimulusDelay);
}

const TextFormat *TrialType::textFormat() const
{
	return(_textFmt);
}

const TextFormat *TrialType::textFormat2() const
{
	return(_textFmt2);
}

Tokenizer::TriggerType TrialType::trigger() const
{
	return(_trigger);
}
